#include <stdio.h>
#include <sybfront.h>
#include <sybdb.h>
#include "sybdbex.h"
#include <wiscbench.h>

/*
 * wisc_query15.c
 *
 * Wisconsin Benchmark - Join A SelB Non_clustered index
 *                                
 *	NOTE: if -n used, then for valid results, this query must be run
 *		before running "wisc_index"
 *
 * Implemented by David DeWitt and Michael Franklin
 * 
 * Copyright (c) 1991 David DeWitt and Michael Franklin
 *
 */

main (argc,argv)
    char	**argv;

{
    /* sybase specific variables */
    DBPROCESS       *dbproc;
    RETCODE        result_code; /* to hold the results of dbresults(). */
    STATUS         row_code;    /* to hold the results of dbnextrow(). */

    char	*username;
    char	*passwd;
    char	*optstring;
    extern 	char *optarg;
    extern 	int optind, opterr;
    int		c;
    int		errflg;

    int		i;
    int		lower,upper; /* bounds for selection predicate */
    int		range;  /* range of predicate */
    float	selFactor; /* selectivity factor */
    int		tupCount; /* number of tuples in the relation */

    char	query[1000];
    int 	starttime, endtime, totaltime;
    int 	numtrials,totalrows,trialrows;

    username = passwd = NULL;
    errflg = 0;
    /* process the command line arguments */
    while ((c = getopt(argc,argv,"u:p:")) != EOF)
    {
	switch (c) {
	case 'u':	// get user name
	    username = optarg; 
	    break;
	case 'p':	// get user passwd
	    passwd = optarg;
	    break;
	case '?':
	    errflg++;
	}
	if (errflg) 
	{
	     usage();
	     exit (2);
	}
    }
    fprintf(stderr,"\nQuery 15 (Non-ClusterIx, JoinAselB)...\n");

    /* next open the database */
    dbproc = open_db(DATABASE, username, passwd,FALSE);

    /* run query 15 - Join A Sel B */

    /* set base constants */
    selFactor = 0.10; /* 10% selectivity factor on B */
    tupCount = BASERELSIZE; /* from wiscbench.h */
    range = ((tupCount+1)*selFactor)-1; /* calculate range for predicates */

    numtrials = 0;
    totaltime = 0;
    totalrows = 0;

    for (i=0;i<JOINREPEATCNT/2;i++)
    {
	/* run JOINREPEATCNT queries two at a time */

	create_join_temp(dbproc,"TMP");
	/* calculate lower and upper ranges of the predicates */
	do 
	{
	    lower = (int) random()%tupCount;
	    upper = lower+range;
	} while (upper >= tupCount);

	sprintf(query,"insert into TMP select * from %s, %s where %s.unique1D = %s.unique1E and (%s.unique1E between %d and %d)", 
	BASERELATION1, BASERELATION2, BASERELATION1, BASERELATION2, 
	BASERELATION2, lower, upper);
	if (VERBOSE) printf("next query is: %s\n",query);
	dbcmd(dbproc,query); /* form the query */

        /* Send the command buffer to SQL Server for execution. */
	starttime = time1000();
	dbsqlexec(dbproc);
        while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) continue;
    	endtime = time1000();

	/* update running statistics */
    	totaltime += (endtime-starttime);
    	trialrows = get_row_count(dbproc);
    	totalrows += trialrows;
    	drop_table(dbproc, "TMP"); /* delete the result relation */
    	print_trial(++numtrials,trialrows,(endtime-starttime)); 

	create_join_temp(dbproc,"TMP");
	/* calculate lower and upper ranges of the predicates */
	do 
	{
	    lower = (int) random()%tupCount;
	    upper = lower+range;
	} while (upper >= tupCount);

	sprintf(query,"insert into TMP select * from %s, %s where %s.unique1E = %s.unique1D and (%s.unique1D between %d and %d)", 
	BASERELATION1, BASERELATION2, BASERELATION2, BASERELATION1, 
	BASERELATION1, lower, upper);

	if (VERBOSE) printf("next query is: %s\n",query);
	dbcmd(dbproc,query); /* form the query */

    	/* Send the command buffer to SQL Server for execution. */
    	starttime = time1000();
    	dbsqlexec(dbproc);
    	while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) continue;;
    	endtime = time1000();

	/* update running statistics */
    	totaltime += (endtime-starttime);
    	trialrows = get_row_count(dbproc);
    	totalrows += trialrows;
    	drop_table(dbproc, "TMP"); /* delete the result relation */
    	print_trial(++numtrials,trialrows,(endtime-starttime)); 
    }
    print_result("Query 15 (Non-ClusterIx, JoinASelB)",
		   numtrials, totalrows, totaltime);
    closedb(dbproc);
    exit(STDEXIT);
}

/* =================================================================== */

usage ()
{
    fprintf (stderr,"\nUsage: wisc_query15 -u username -p passwd\n");
    fprintf (stderr,"\t-p: Show query plan\n");
    exit (1);
}
